"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.readHunspellFiles = exports.createArrayReader = exports.createReader = void 0;
const gensequence_1 = require("gensequence");
const HR = __importStar(require("hunspell-reader"));
const fs = __importStar(require("fs-extra"));
const cspell_trie_lib_1 = require("cspell-trie-lib");
const zlib = __importStar(require("zlib"));
const regHunspellFile = /\.(dic|aff)$/i;
// cspell:word dedupe
const DEDUPE_SIZE = 1000;
const regExMatchComments = /\s*(#|\/\/).*/;
// Readers first match wins
const readers = [
    { test: /\.trie\b/, method: trieFileReader },
    { test: regHunspellFile, method: readHunspellFiles },
];
function findMatchingReader(filename, options) {
    for (const reader of readers) {
        if (reader.test.test(filename)) {
            return reader.method(filename, options);
        }
    }
    return textFileReader(filename, options);
}
async function createReader(filename, options) {
    const baseReader = await findMatchingReader(filename, options);
    return Object.assign(baseReader, {
        [Symbol.iterator]: options.useAnnotation ? baseReader.annotatedWords : baseReader.rawWords,
    });
}
exports.createReader = createReader;
function createArrayReader(lines) {
    const rawWords = () => gensequence_1.genSequence(lines);
    const annotatedWords = () => gensequence_1.genSequence(lines).pipe(_mapText, dedupeAndSort);
    return {
        size: lines.length,
        annotatedWords,
        rawWords,
    };
}
exports.createArrayReader = createArrayReader;
async function readHunspellFiles(filename, options) {
    const dicFile = filename.replace(regHunspellFile, '.dic');
    const affFile = filename.replace(regHunspellFile, '.aff');
    const reader = await HR.IterableHunspellReader.createFromFiles(affFile, dicFile);
    reader.maxDepth = options.maxDepth !== undefined ? options.maxDepth : reader.maxDepth;
    const normalizeAndDedupe = gensequence_1.operators.pipe(_stripCaseAndAccents, dedupeAndSort);
    const rawWords = () => reader.seqWords();
    return {
        size: reader.dic.length,
        annotatedWords() {
            return reader.seqAffWords().pipe(_mapAffWords).pipe(normalizeAndDedupe);
        },
        rawWords,
    };
}
exports.readHunspellFiles = readHunspellFiles;
async function trieFileReader(filename) {
    const trieRoot = cspell_trie_lib_1.importTrie(await readTextFile(filename));
    const trie = new cspell_trie_lib_1.Trie(trieRoot);
    const rawWords = () => trie.words();
    return {
        get size() {
            return trie.size();
        },
        annotatedWords: rawWords,
        rawWords,
    };
}
function readTextFile(filename) {
    const lines = fs
        .readFile(filename)
        .then((buffer) => (/\.gz$/.test(filename) ? zlib.gunzipSync(buffer) : buffer))
        .then((buffer) => buffer.toString('utf8'))
        .then((content) => content.split(/\r?\n/g));
    return lines;
}
async function textFileReader(filename, _options) {
    const lines = await readTextFile(filename);
    return createArrayReader(lines);
}
const _mapText = gensequence_1.operators.pipe(_comments, _compoundBegin, _compoundEnd, _stripCaseAndAccents);
function* _comments(lines) {
    for (const line of lines) {
        const w = line.replace(regExMatchComments, '').trim();
        if (w)
            yield w;
    }
}
function* _compoundEnd(lines) {
    for (const line of lines) {
        if (line[0] !== cspell_trie_lib_1.OPTIONAL_COMPOUND) {
            yield line;
            continue;
        }
        const w = line.slice(1);
        yield w;
        yield cspell_trie_lib_1.COMPOUND + w;
    }
}
function* _compoundBegin(lines) {
    for (const line of lines) {
        if (line[line.length - 1] !== cspell_trie_lib_1.OPTIONAL_COMPOUND) {
            yield line;
            continue;
        }
        const w = line.slice(0, -1);
        yield w;
        yield w + cspell_trie_lib_1.COMPOUND;
    }
}
function* _stripCaseAndAccents(words) {
    for (const word of words) {
        // Words are normalized to the compact format: e + ` => è
        yield word.normalize();
        // covert to lower case and strip accents.
        const n = word
            .toLowerCase()
            .normalize('NFD')
            .replace(/[\u0300-\u036f]/g, '');
        // All words are added for case-insensitive searches.
        // It is a space / speed trade-off. In this case, speed is more important.
        yield cspell_trie_lib_1.NORMALIZED + n;
    }
}
function* dedupeAndSort(words) {
    const buffer = [];
    function sortDedupeClear() {
        const s = new Set(buffer.sort());
        buffer.length = 0;
        return s;
    }
    for (const word of words) {
        buffer.push(word);
        if (buffer.length >= DEDUPE_SIZE) {
            yield* sortDedupeClear();
        }
    }
    yield* sortDedupeClear();
}
function* _mapAffWords(affWords) {
    for (const affWord of affWords) {
        const { word, flags } = affWord;
        const compound = flags.isCompoundForbidden ? '' : cspell_trie_lib_1.COMPOUND;
        const forbid = flags.isForbiddenWord ? cspell_trie_lib_1.FORBID : '';
        if (!forbid) {
            if (flags.canBeCompoundBegin)
                yield word + compound;
            if (flags.canBeCompoundEnd)
                yield compound + word;
            if (flags.canBeCompoundMiddle)
                yield compound + word + compound;
            if (!flags.isOnlyAllowedInCompound)
                yield word;
        }
        else {
            yield forbid + word;
        }
    }
}
//# sourceMappingURL=Reader.js.map